var EXPORTED_SYMBOLS = ["GlobalStoreCache"];


var Ci = Components.interfaces;
var Cc = Components.classes;
var Cu = Components.utils;


Cu.import("resource://extrabux/modules/logging.jsm");
var nativeJSON = Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);


Cu.import("resource://extrabux/modules/underscore.jsm");
Cu.import("resource://extrabux/modules/inheritFrom.jsm");
Cu.import("resource://extrabux/modules/event.jsm");
Cu.import("resource://extrabux/modules/storage.jsm");


Cu.import("resource://extrabux/modules/xmlHttpPaginator.jsm");
Cu.import("resource://extrabux/modules/config.jsm");
Cu.import("resource://extrabux/modules/utils.jsm");
var _txt = Utils.text;



// Base URL for all Store-API-related requests.
var baseURL = Config.baseURL + "store/";


function StoreCache() {
  this.cachedList = [];

  /* Create Events. */
  this.listLoaded = new Event(this);
  this.listLoading = new Event(this);
  this.listError = new Event(this);

  this.storage = new LocalStorage();

  this.initialized = false;

  this.timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
}


/* Inherit the HTTP voodoo. */
InheritFrom(XMLHTTPPaginator, StoreCache);


StoreCache.prototype.initializeUpdating = function() {
  if (this.initialized) {
    return;
  }
  this.initialized = true;

  var timeLeft = this.calculateTimeToNextUpdate();
  this.checkUpdateWorker(timeLeft);
};


StoreCache.prototype.calculateTimeToNextUpdate = function() {
  var prefs = Utils.getPrefs();
  var lastUpdateTimestamp = parseInt(prefs.getCharPref("lastUpdateTimestamp"), 10);
  if (lastUpdateTimestamp === 0) {
    return 0;
  }

  var elapsedSinceLastUpdate = Date.now() - lastUpdateTimestamp;
  var timeToNextUpdate =
    Math.max(Config.storesListUpdateIntervalMs - elapsedSinceLastUpdate, 0);

  return timeToNextUpdate;
};

StoreCache.prototype.checkUpdateWorker = function(timeToNextUpdate) {
  var self = this;

  var event = {
    notify: function(timer) {
      self.reloadCache();
      timer.initWithCallback(
        event, Config.storesListUpdateIntervalMs, Ci.nsITimer.TYPE_ONE_SHOT);
    }
  };

  this.timer.initWithCallback(event, timeToNextUpdate, Ci.nsITimer.TYPE_ONE_SHOT);
};


/**
 * Download the list of stores from the server.
 */
StoreCache.prototype._downloadList = function(params, maxResults, callbackObj) {
  // This is just a little trick (turn the ugly method into a nicer one).
  this.superClass._downloadList(
    baseURL, params, "total_stores", "stores",
    "store", maxResults, callbackObj);
};


StoreCache.prototype.filterRestrictedStores = function(storesList) {
  return _.reject(storesList, function(store){
    var discontinued = parseInt(_txt(store.plugin_restricted), 10);
    return (discontinued == 1);
  });
};

/*
 *  Reload the cached stores list.
 *  @param fromRemoteSource: If true, force downloading new list from
*       Internet. Useful for updating the list every 24-hours, for example.
 */
StoreCache.prototype.reloadCache = function() {

  logger.debug("StoreCache -> Reloading cache remotely...");

  this.listLoading.notify();

  var key = "stores";

  var self = this;

  // Download the list of stores from Internet.
  this._downloadList(
    {sort: "rank", count: 50},
    0, // Set to 0 means ignore maxResults.
    {"on": {
    "success": function(storesList, finished) {
      storesList = self.filterRestrictedStores(storesList);
      self.cachedList = storesList;
      if (finished) {
        self.storage.setAsync(key, storesList, {"on": {
          "success": function() {
            var prefs = Utils.getPrefs();
            var now = Date.now();
            prefs.setCharPref("lastUpdateTimestamp", now.toString());
            logger.debug("StoreModel -> reloadCache -> Stored list of " +
              storesList.length + " stores to local storage.");
          },
          "error": function(reason) {
            logger.error("Could not store stores list. Reason: " + reason);
          }
        }});
      }
      try {
        self.listLoaded.notify(storesList);
      } catch(e) {
        logger.error("StoreCache -> reloadCache(remotely) -> notifying: " + e);
      }
    },
    "error": function() {
      self.listError.notify("Failed to load stores list.");
    }
  }});
};


StoreCache.prototype.reloadCacheLocally = function() {

  logger.debug("StoreCache -> Reloading cache locally...");

  this.listLoading.notify();

  var key = "stores";

  var self = this;

  // Try to get the list of stores from local storage first.
  this.storage.getAsync(key, {"on": {
    "success": function(storesList) {
      if ( ! storesList ) {
        // No results => We have to load the list from Internet.
        logger.debug("StoreModel -> reloadCache -> getAsync (localstorage) -> " +
          "no results");
      } else {
        storesList = self.filterRestrictedStores(storesList);
        // Yahoo! We've got what we needed from local storage.
        logger.debug(_.sprintf("StoreModel -> reloadCache -> " +
          "getAsync (localstorage) -> fetched %d stores " +
          "list from local storage.", storesList.length));
        self.cachedList = storesList;
        try {
          self.listLoaded.notify(storesList);
        } catch (e) {
          logger.error("StoreCache -> reloadCache(locally) -> notifying: " + e);
        }
      }
    },
    "error": function() {
      self.listError.notify("Failed to load stores list.");
    }
  }});
};


var GlobalStoreCache = new StoreCache();
